% This script allows you to check to what extent the simulation does the sequence of steps you want it to make

%  Here you can encode your "Expectation": Which quantum states are supposed to be realised when:

% before preparation
cA = [1 0];
cB = [1 0];
initialstate = [cA(1)*cB(1)  cA(1)*cB(2)  cA(2)*cB(1)  cA(2)*cB(2)]; 

% after preparation
phiA=0.3*pi;
cA = [exp(-i*phiA/2)  i*exp(i*phiA/2)]/sqrt(2);
phiB=-pi/2;
cB = [exp(-i*phiB/2)  i*exp(i*phiB/2)]/sqrt(2);

after_register_preparation = [cA(1)*cB(1)  cA(1)*cB(2)  cA(2)*cB(1)  cA(2)*cB(2)]; 
rhoprepared = after_register_preparation'*after_register_preparation;

% after gate
after_gate_operation = [after_register_preparation(1) after_register_preparation(2) after_register_preparation(3) -after_register_preparation(4)];
rhoaftergate = after_gate_operation'*after_gate_operation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Here we extract the simulation density matrices, at some specific times, corresponding to the above

rho = mean_rho_re_1 + i*mean_rho_im_1;
rhomat= zeros(length(t_1),4,4);

for tt=1:length(t_1),
%reshape density matrix into an actual matrix:
   rhomat(tt,:,:) = reshape(squeeze(rho(tt,:,:,:,:)),4,4);
end

% chose this time after your preparation is complete
t_afterprep =1.8;
% chose this time after your gate is complete
t_aftergate = t_1(end);

[dump tind_afterprep] = min(abs(t_1 -t_afterprep));
[dump tind_aftergate] = min(abs(t_1 - t_aftergate));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot(2,4,1)
bar3(real(rhoprepared))
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after prep, target Re'])

subplot(2,4,2)
bar3(imag(rhoprepared))
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after prep, target Im'])

subplot(2,4,3)
bar3(real(squeeze(rhomat(tind_afterprep,:,:))))
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after prep, actual Re'])

subplot(2,4,4)
bar3(imag(squeeze(rhomat(tind_afterprep,:,:))))
xlabel(['A'])
ylabel(['B'])
zlabel(['Im'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after prep, actual Im'])

subplot(2,4,5)
bar3(real(rhoaftergate))
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after gate, target Re'])

subplot(2,4,6)
bar3(imag(rhoaftergate))
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after gate, target Im'])

subplot(2,4,7)
bar3(real(squeeze(rhomat(tind_aftergate,:,:))))
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after gate, actual Re'])

subplot(2,4,8)
bar3(imag(squeeze(rhomat(tind_aftergate,:,:))))
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks(1:4)
xticklabels({'00','01','10','11'})
yticks(1:4)
yticklabels({'00','01','10','11'})
title(['after gate, actual Im'])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lw=2;
fs=20;

lineobj = findobj('type', 'Line');
set(lineobj,'LineWidth', lw);

patchobj = findobj('type', 'Patch');
set(patchobj,'LineWidth', lw);

textobj = findobj('type', 'axes');
set(textobj,'LineWidth', lw);
set(textobj, 'fontsize', fs);
set(textobj, 'FontName', 'Times');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  